@@ -2,7 +2,7 @@
 
 /*  vcfcall.c -- SNP/indel variant calling from VCF/BCF.
 
-    Copyright (C) 2013-2014 Genome Research Ltd.
+    Copyright (C) 2013-2016 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -70,7 +70,7 @@ void error(const char *format, ...);
 typedef struct
 {
     int flag;   // combination of CF_* flags above
-    int output_type, n_threads;
+    int output_type, n_threads, record_cmd_line;
     htsFile *bcf_in, *out_fh;
     char *bcf_fname, *output_fname;
     char **samples;             // for subsampling and ploidy
@@ -177,6 +177,11 @@ static ploidy_predef_t ploidy_predefs[] =
           "*  * *     M 1\n"
           "*  * *     F 0\n"
     },
+    { .alias  = "1",
+      .about  = "Treat all samples as haploid",
+      .ploidy =
+          "*  * *     * 1\n"
+    },
     {
         .alias  = NULL,
         .about  = NULL,
@@ -290,7 +295,7 @@ static void set_samples(args_t *args, const char *fn, int is_file)
         char x = *se, *xptr = se; *se = 0;
 
         int ismpl = bcf_hdr_id2int(args->aux.hdr, BCF_DT_SAMPLE, ss);
-        if ( ismpl < 0 ) { fprintf(pysamerr,"Warning: No such sample in the VCF: %s\n",ss); continue; }
+        if ( ismpl < 0 ) { fprintf(pysam_stderr,"Warning: No such sample in the VCF: %s\n",ss); continue; }
 
         ss = se+1;
         while ( *ss && isspace(*ss) ) ss++;
@@ -383,7 +388,7 @@ static void init_data(args_t *args)
     if ( args->regions )
     {
         if ( bcf_sr_set_regions(args->aux.srs, args->regions, args->regions_is_file)<0 )
-            error("Failed to read the targets: %s\n", args->regions);
+            error("Failed to read the regions: %s\n", args->regions);
     }
 
     if ( !bcf_sr_add_reader(args->aux.srs, args->bcf_fname) ) error("Failed to open %s: %s\n", args->bcf_fname,bcf_sr_strerror(args->aux.srs->errnum));
@@ -396,11 +401,23 @@ static void init_data(args_t *args)
         if ( args->aux.flag&CALL_CONSTR_TRIO )
         {
             if ( 3*args->aux.nfams!=args->nsamples ) error("Expected only trios in %s, sorry!\n", args->samples_fname);
-            fprintf(pysamerr,"Detected %d samples in %d trio families\n", args->nsamples,args->aux.nfams);
+            fprintf(pysam_stderr,"Detected %d samples in %d trio families\n", args->nsamples,args->aux.nfams);
         }
+    }
+    if ( args->ploidy  )
+    {
         args->nsex = ploidy_nsex(args->ploidy);
         args->sex2ploidy = (int*) calloc(args->nsex,sizeof(int));
         args->sex2ploidy_prev = (int*) calloc(args->nsex,sizeof(int));
+        if ( !args->nsamples )
+        {
+            args->nsamples = bcf_hdr_nsamples(args->aux.hdr);
+            args->sample2sex = (int*) malloc(sizeof(int)*args->nsamples);
+            for (i=0; i<args->nsamples; i++) args->sample2sex[i] = 0;
+        }
+    }
+    if ( args->nsamples )
+    {
         args->aux.ploidy = (uint8_t*) malloc(args->nsamples);
         for (i=0; i<args->nsamples; i++) args->aux.ploidy[i] = 2;
         for (i=0; i<args->nsex; i++) args->sex2ploidy_prev[i] = 2;
@@ -420,9 +437,12 @@ static void init_data(args_t *args)
     else
     {
         args->aux.hdr = bcf_hdr_dup(bcf_sr_get_header(args->aux.srs,0));
-        for (i=0; i<args->nsamples; i++)
-            if ( bcf_hdr_id2int(args->aux.hdr,BCF_DT_SAMPLE,args->samples[i])<0 )
-                error("No such sample: %s\n", args->samples[i]);
+        if ( args->samples )
+        {
+            for (i=0; i<args->nsamples; i++)
+                if ( bcf_hdr_id2int(args->aux.hdr,BCF_DT_SAMPLE,args->samples[i])<0 )
+                    error("No such sample: %s\n", args->samples[i]);
+        }
     }
 
     args->out_fh = hts_open(args->output_fname, hts_bcf_wmode(args->output_type));
@@ -441,7 +461,7 @@ static void init_data(args_t *args)
     bcf_hdr_remove(args->aux.hdr, BCF_HL_INFO, "QS");
     bcf_hdr_remove(args->aux.hdr, BCF_HL_INFO, "I16");
 
-    bcf_hdr_append_version(args->aux.hdr, args->argc, args->argv, "bcftools_call");
+    if (args->record_cmd_line) bcf_hdr_append_version(args->aux.hdr, args->argc, args->argv, "bcftools_call");
     bcf_hdr_write(args->out_fh, args->aux.hdr);
 
     if ( args->flag&CF_INS_MISSED ) init_missed_line(args);
@@ -453,7 +473,10 @@ static void destroy_data(args_t *args)
     else if ( args->flag & CF_MCALL ) mcall_destroy(&args->aux);
     else if ( args->flag & CF_QCALL ) qcall_destroy(&args->aux);
     int i;
-    for (i=0; i<args->nsamples; i++) free(args->samples[i]);
+    if ( args->samples )
+    {
+        for (i=0; i<args->nsamples; i++) free(args->samples[i]);
+    }
     if ( args->aux.fams )
     {
         for (i=0; i<args->aux.nfams; i++) free(args->aux.fams[i].name);
@@ -507,7 +530,7 @@ static int parse_format_flag(const char *str)
         else if ( !strncasecmp(ss,"GP",se-ss) ) flag |= CALL_FMT_GP;
         else
         {
-            fprintf(pysamerr,"Could not parse \"%s\"\n", str);
+            fprintf(pysam_stderr,"Could not parse \"%s\"\n", str);
             exit(1);
         }
         if ( !*se ) break;
@@ -548,23 +571,23 @@ ploidy_t *init_ploidy(char *alias)
 
     if ( !pld->alias )
     {
-        fprintf(pysamerr,"Predefined ploidies:\n");
+        fprintf(pysam_stderr,"Predefined ploidies:\n");
         pld = ploidy_predefs;
         while ( pld->alias )
         {
-            fprintf(pysamerr,"%s\n   .. %s\n\n", pld->alias,pld->about);
+            fprintf(pysam_stderr,"%s\n   .. %s\n\n", pld->alias,pld->about);
             if ( detailed )
-                fprintf(pysamerr,"%s\n", pld->ploidy);
+                fprintf(pysam_stderr,"%s\n", pld->ploidy);
             pld++;
         }
-        fprintf(pysamerr,"Run as --ploidy <alias> (e.g. --ploidy GRCh37).\n");
-        fprintf(pysamerr,"To see the detailed ploidy definition, append a question mark (e.g. --ploidy GRCh37?).\n");
-        fprintf(pysamerr,"\n");
+        fprintf(pysam_stderr,"Run as --ploidy <alias> (e.g. --ploidy GRCh37).\n");
+        fprintf(pysam_stderr,"To see the detailed ploidy definition, append a question mark (e.g. --ploidy GRCh37?).\n");
+        fprintf(pysam_stderr,"\n");
         exit(-1);
     }
     else if ( detailed )
     {
-        fprintf(pysamerr,"%s", pld->ploidy);
+        fprintf(pysam_stderr,"%s", pld->ploidy);
         exit(-1);
     }
     return ploidy_init_string(pld->ploidy,2);
@@ -572,51 +595,52 @@ ploidy_t *init_ploidy(char *alias)
 
 static void usage(args_t *args)
 {
-    fprintf(pysamerr, "\n");
-    fprintf(pysamerr, "About:   SNP/indel variant calling from VCF/BCF. To be used in conjunction with samtools mpileup.\n");
-    fprintf(pysamerr, "         This command replaces the former \"bcftools view\" caller. Some of the original\n");
-    fprintf(pysamerr, "         functionality has been temporarily lost in the process of transition to htslib,\n");
-    fprintf(pysamerr, "         but will be added back on popular demand. The original calling model can be\n");
-    fprintf(pysamerr, "         invoked with the -c option.\n");
-    fprintf(pysamerr, "Usage:   bcftools call [options] <in.vcf.gz>\n");
-    fprintf(pysamerr, "\n");
-    fprintf(pysamerr, "File format options:\n");
-    fprintf(pysamerr, "   -o, --output <file>             write output to a file [standard output]\n");
-    fprintf(pysamerr, "   -O, --output-type <b|u|z|v>     output type: 'b' compressed BCF; 'u' uncompressed BCF; 'z' compressed VCF; 'v' uncompressed VCF [v]\n");
-    fprintf(pysamerr, "       --ploidy <assembly>[?]      predefined ploidy, 'list' to print available settings, append '?' for details\n");
-    fprintf(pysamerr, "       --ploidy-file <file>        space/tab-delimited list of CHROM,FROM,TO,SEX,PLOIDY\n");
-    fprintf(pysamerr, "   -r, --regions <region>          restrict to comma-separated list of regions\n");
-    fprintf(pysamerr, "   -R, --regions-file <file>       restrict to regions listed in a file\n");
-    fprintf(pysamerr, "   -s, --samples <list>            list of samples to include [all samples]\n");
-    fprintf(pysamerr, "   -S, --samples-file <file>       PED file or a file with an optional column with sex (see man page for details) [all samples]\n");
-    fprintf(pysamerr, "   -t, --targets <region>          similar to -r but streams rather than index-jumps\n");
-    fprintf(pysamerr, "   -T, --targets-file <file>       similar to -R but streams rather than index-jumps\n");
-    fprintf(pysamerr, "       --threads <int>             number of extra output compression threads [0]\n");
-    fprintf(pysamerr, "\n");
-    fprintf(pysamerr, "Input/output options:\n");
-    fprintf(pysamerr, "   -A, --keep-alts                 keep all possible alternate alleles at variant sites\n");
-    fprintf(pysamerr, "   -f, --format-fields <list>      output format fields: GQ,GP (lowercase allowed) []\n");
-    fprintf(pysamerr, "   -g, --gvcf <int>,[...]          group non-variant sites into gVCF blocks by minimum per-sample DP\n");
-    fprintf(pysamerr, "   -i, --insert-missed             output also sites missed by mpileup but present in -T\n");
-    fprintf(pysamerr, "   -M, --keep-masked-ref           keep sites with masked reference allele (REF=N)\n");
-    fprintf(pysamerr, "   -V, --skip-variants <type>      skip indels/snps\n");
-    fprintf(pysamerr, "   -v, --variants-only             output variant sites only\n");
-    fprintf(pysamerr, "\n");
-    fprintf(pysamerr, "Consensus/variant calling options:\n");
-    fprintf(pysamerr, "   -c, --consensus-caller          the original calling method (conflicts with -m)\n");
-    fprintf(pysamerr, "   -C, --constrain <str>           one of: alleles, trio (see manual)\n");
-    fprintf(pysamerr, "   -m, --multiallelic-caller       alternative model for multiallelic and rare-variant calling (conflicts with -c)\n");
-    fprintf(pysamerr, "   -n, --novel-rate <float>,[...]  likelihood of novel mutation for constrained trio calling, see man page for details [1e-8,1e-9,1e-9]\n");
-    fprintf(pysamerr, "   -p, --pval-threshold <float>    variant if P(ref|D)<FLOAT with -c [0.5]\n");
-    fprintf(pysamerr, "   -P, --prior <float>             mutation rate (use bigger for greater sensitivity) [1.1e-3]\n");
+    fprintf(pysam_stderr, "\n");
+    fprintf(pysam_stderr, "About:   SNP/indel variant calling from VCF/BCF. To be used in conjunction with samtools mpileup.\n");
+    fprintf(pysam_stderr, "         This command replaces the former \"bcftools view\" caller. Some of the original\n");
+    fprintf(pysam_stderr, "         functionality has been temporarily lost in the process of transition to htslib,\n");
+    fprintf(pysam_stderr, "         but will be added back on popular demand. The original calling model can be\n");
+    fprintf(pysam_stderr, "         invoked with the -c option.\n");
+    fprintf(pysam_stderr, "Usage:   bcftools call [options] <in.vcf.gz>\n");
+    fprintf(pysam_stderr, "\n");
+    fprintf(pysam_stderr, "File format options:\n");
+    fprintf(pysam_stderr, "       --no-version                do not append version and command line to the header\n");
+    fprintf(pysam_stderr, "   -o, --output <file>             write output to a file [standard output]\n");
+    fprintf(pysam_stderr, "   -O, --output-type <b|u|z|v>     output type: 'b' compressed BCF; 'u' uncompressed BCF; 'z' compressed VCF; 'v' uncompressed VCF [v]\n");
+    fprintf(pysam_stderr, "       --ploidy <assembly>[?]      predefined ploidy, 'list' to print available settings, append '?' for details\n");
+    fprintf(pysam_stderr, "       --ploidy-file <file>        space/tab-delimited list of CHROM,FROM,TO,SEX,PLOIDY\n");
+    fprintf(pysam_stderr, "   -r, --regions <region>          restrict to comma-separated list of regions\n");
+    fprintf(pysam_stderr, "   -R, --regions-file <file>       restrict to regions listed in a file\n");
+    fprintf(pysam_stderr, "   -s, --samples <list>            list of samples to include [all samples]\n");
+    fprintf(pysam_stderr, "   -S, --samples-file <file>       PED file or a file with an optional column with sex (see man page for details) [all samples]\n");
+    fprintf(pysam_stderr, "   -t, --targets <region>          similar to -r but streams rather than index-jumps\n");
+    fprintf(pysam_stderr, "   -T, --targets-file <file>       similar to -R but streams rather than index-jumps\n");
+    fprintf(pysam_stderr, "       --threads <int>             number of extra output compression threads [0]\n");
+    fprintf(pysam_stderr, "\n");
+    fprintf(pysam_stderr, "Input/output options:\n");
+    fprintf(pysam_stderr, "   -A, --keep-alts                 keep all possible alternate alleles at variant sites\n");
+    fprintf(pysam_stderr, "   -f, --format-fields <list>      output format fields: GQ,GP (lowercase allowed) []\n");
+    fprintf(pysam_stderr, "   -g, --gvcf <int>,[...]          group non-variant sites into gVCF blocks by minimum per-sample DP\n");
+    fprintf(pysam_stderr, "   -i, --insert-missed             output also sites missed by mpileup but present in -T\n");
+    fprintf(pysam_stderr, "   -M, --keep-masked-ref           keep sites with masked reference allele (REF=N)\n");
+    fprintf(pysam_stderr, "   -V, --skip-variants <type>      skip indels/snps\n");
+    fprintf(pysam_stderr, "   -v, --variants-only             output variant sites only\n");
+    fprintf(pysam_stderr, "\n");
+    fprintf(pysam_stderr, "Consensus/variant calling options:\n");
+    fprintf(pysam_stderr, "   -c, --consensus-caller          the original calling method (conflicts with -m)\n");
+    fprintf(pysam_stderr, "   -C, --constrain <str>           one of: alleles, trio (see manual)\n");
+    fprintf(pysam_stderr, "   -m, --multiallelic-caller       alternative model for multiallelic and rare-variant calling (conflicts with -c)\n");
+    fprintf(pysam_stderr, "   -n, --novel-rate <float>,[...]  likelihood of novel mutation for constrained trio calling, see man page for details [1e-8,1e-9,1e-9]\n");
+    fprintf(pysam_stderr, "   -p, --pval-threshold <float>    variant if P(ref|D)<FLOAT with -c [0.5]\n");
+    fprintf(pysam_stderr, "   -P, --prior <float>             mutation rate (use bigger for greater sensitivity) [1.1e-3]\n");
 
     // todo (and more)
-    // fprintf(pysamerr, "\nContrast calling and association test options:\n");
-    // fprintf(pysamerr, "       -1 INT    number of group-1 samples [0]\n");
-    // fprintf(pysamerr, "       -C FLOAT  posterior constrast for LRT<FLOAT and P(ref|D)<0.5 [%g]\n", args->aux.min_lrt);
-    // fprintf(pysamerr, "       -U INT    number of permutations for association testing (effective with -1) [0]\n");
-    // fprintf(pysamerr, "       -X FLOAT  only perform permutations for P(chi^2)<FLOAT [%g]\n", args->aux.min_perm_p);
-    fprintf(pysamerr, "\n");
+    // fprintf(pysam_stderr, "\nContrast calling and association test options:\n");
+    // fprintf(pysam_stderr, "       -1 INT    number of group-1 samples [0]\n");
+    // fprintf(pysam_stderr, "       -C FLOAT  posterior constrast for LRT<FLOAT and P(ref|D)<0.5 [%g]\n", args->aux.min_lrt);
+    // fprintf(pysam_stderr, "       -U INT    number of permutations for association testing (effective with -1) [0]\n");
+    // fprintf(pysam_stderr, "       -X FLOAT  only perform permutations for P(chi^2)<FLOAT [%g]\n", args->aux.min_perm_p);
+    fprintf(pysam_stderr, "\n");
     exit(-1);
 }
 
@@ -636,6 +660,7 @@ int main_vcfcall(int argc, char *argv[])
     args.output_fname   = "-";
     args.output_type    = FT_VCF;
     args.n_threads = 0;
+    args.record_cmd_line = 1;
     args.aux.trio_Pm_SNPs = 1 - 1e-8;
     args.aux.trio_Pm_ins  = args.aux.trio_Pm_del  = 1 - 1e-9;
 
@@ -670,6 +695,7 @@ int main_vcfcall(int argc, char *argv[])
         {"ploidy-file",required_argument,NULL,2},
         {"chromosome-X",no_argument,NULL,'X'},
         {"chromosome-Y",no_argument,NULL,'Y'},
+        {"no-version",no_argument,NULL,8},
         {NULL,0,NULL,0}
     };
 
@@ -680,8 +706,8 @@ int main_vcfcall(int argc, char *argv[])
         {
             case  2 : ploidy_fname = optarg; break;
             case  1 : ploidy = optarg; break;
-            case 'X': ploidy = "X"; fprintf(pysamerr,"Warning: -X will be deprecated, please use --ploidy instead.\n"); break;
-            case 'Y': ploidy = "Y"; fprintf(pysamerr,"Warning: -Y will be deprecated, please use --ploidy instead.\n"); break;
+            case 'X': ploidy = "X"; fprintf(pysam_stderr,"Warning: -X will be deprecated, please use --ploidy instead.\n"); break;
+            case 'Y': ploidy = "Y"; fprintf(pysam_stderr,"Warning: -Y will be deprecated, please use --ploidy instead.\n"); break;
             case 'f': args.aux.output_tags |= parse_format_flag(optarg); break;
             case 'M': args.flag &= ~CF_ACGT_ONLY; break;     // keep sites where REF is N
             case 'N': args.flag |= CF_ACGT_ONLY; break;      // omit sites where first base in REF is N (the new default)
@@ -729,6 +755,7 @@ int main_vcfcall(int argc, char *argv[])
             case 's': args.samples_fname = optarg; break;
             case 'S': args.samples_fname = optarg; args.samples_is_file = 1; break;
             case  9 : args.n_threads = strtol(optarg, 0, 0); break;
+            case  8 : args.record_cmd_line = 0; break;
             default: usage(&args);
         }
     }
@@ -745,7 +772,7 @@ int main_vcfcall(int argc, char *argv[])
 
     if ( !ploidy_fname && !ploidy )
     {
-        fprintf(pysamerr,"Note: Neither --ploidy nor --ploidy-file given, assuming all sites are diploid\n");
+        fprintf(pysam_stderr,"Note: Neither --ploidy nor --ploidy-file given, assuming all sites are diploid\n");
         args.ploidy = ploidy_init_string("",2);
     }
 
