@@ -120,7 +120,7 @@ typedef struct
     htsFile *out_fh;
     bcf_hdr_t *out_hdr;
     char **argv;
-    int argc, n_threads;
+    int argc, n_threads, record_cmd_line;
 }
 args_t;
 
@@ -451,8 +451,8 @@ void merge_headers(bcf_hdr_t *hw, const bcf_hdr_t *hr, const char *clash_prefix,
 
 void debug_als(char **als, int nals)
 {
-    int k; for (k=0; k<nals; k++) fprintf(pysamerr,"%s ", als[k]);
-    fprintf(pysamerr,"\n");
+    int k; for (k=0; k<nals; k++) fprintf(pysam_stderr,"%s ", als[k]);
+    fprintf(pysam_stderr,"\n");
 }
 
 /**
@@ -534,7 +534,7 @@ char **merge_alleles(char **a, int na, int *map, char **b, int *nb, int *mb)
     {
         if ( strncasecmp(a[0],b[0],rla<rlb?rla:rlb) )
         {
-            fprintf(pysamerr, "The REF prefixes differ: %s vs %s (%d,%d)\n", a[0],b[0],rla,rlb);
+            fprintf(pysam_stderr, "The REF prefixes differ: %s vs %s (%d,%d)\n", a[0],b[0],rla,rlb);
             return NULL;
         }
         // Different case, change to uppercase
@@ -657,13 +657,13 @@ void maux_reset(maux_t *ma)
 }
 void maux_debug(maux_t *ma, int ir, int ib)
 {
-    printf("[%d,%d]\t", ir,ib);
+    fprintf(pysam_stdout, "[%d,%d]\t", ir,ib);
     int i;
     for (i=0; i<ma->nals; i++)
     {
-        printf(" %s [%d]", ma->als[i], ma->cnt[i]);
+        fprintf(pysam_stdout, " %s [%d]", ma->als[i], ma->cnt[i]);
     }
-    printf("\n");
+    fprintf(pysam_stdout, "\n");
 }
 
 void merge_chrom2qual(args_t *args, bcf1_t *out)
@@ -860,7 +860,7 @@ int copy_string_field(char *src, int isrc, int src_len, kstring_t *dst, int idst
     }
     if ( ith_src!=isrc ) return -1; // requested field not found
     int end_src = start_src;
-    while ( end_src<src_len && src[end_src]!=',' ) end_src++;
+    while ( end_src<src_len && src[end_src] && src[end_src]!=',' ) end_src++;
 
     int nsrc_cpy = end_src - start_src;
     if ( nsrc_cpy==1 && src[start_src]=='.' ) return 0;   // don't write missing values, dst is already initialized
@@ -946,7 +946,7 @@ static void merge_AGR_info_tag(bcf_hdr_t *hdr, bcf1_t *line, bcf_info_t *info, i
                 case BCF_BT_INT16: BRANCH(int16_t, *src==bcf_int16_missing, *src==bcf_int16_vector_end, int); break;
                 case BCF_BT_INT32: BRANCH(int32_t, *src==bcf_int32_missing, *src==bcf_int32_vector_end, int); break;
                 case BCF_BT_FLOAT: BRANCH(float,   bcf_float_is_missing(*src), bcf_float_is_vector_end(*src), float); break;
-                default: fprintf(pysamerr,"TODO: %s:%d .. info->type=%d\n", __FILE__,__LINE__, info->type); exit(1);
+                default: fprintf(pysam_stderr,"TODO: %s:%d .. info->type=%d\n", __FILE__,__LINE__, info->type); exit(1);
             }
             #undef BRANCH
         }
@@ -976,7 +976,7 @@ static void merge_AGR_info_tag(bcf_hdr_t *hdr, bcf1_t *line, bcf_info_t *info, i
                 case BCF_BT_INT16: BRANCH(int16_t, src[kori]==bcf_int16_missing, src[kori]==bcf_int16_vector_end, int); break;
                 case BCF_BT_INT32: BRANCH(int32_t, src[kori]==bcf_int32_missing, src[kori]==bcf_int32_vector_end, int); break;
                 case BCF_BT_FLOAT: BRANCH(float,   bcf_float_is_missing(src[kori]), bcf_float_is_vector_end(src[kori]), float); break;
-                default: fprintf(pysamerr,"TODO: %s:%d .. info->type=%d\n", __FILE__,__LINE__, info->type); exit(1);
+                default: fprintf(pysam_stderr,"TODO: %s:%d .. info->type=%d\n", __FILE__,__LINE__, info->type); exit(1);
             }
             #undef BRANCH
         }
@@ -1556,7 +1556,7 @@ void shake_buffer(maux_t *maux, int ir, int pos)
     if ( !reader->buffer ) return;
 
     int i;
-    // FILE *fp = stdout;
+    // FILE *fp = pysam_stdout;
     // fprintf(fp,"<going to shake> nbuf=%d\t", reader->nbuffer); for (i=0; i<reader->nbuffer; i++) fprintf(fp," %d", skip[i]); fprintf(fp,"\n");
     // debug_buffer(fp,reader);
     // fprintf(fp,"--\n");
@@ -1641,43 +1641,43 @@ void debug_maux(args_t *args, int pos, int var_type)
     maux_t *maux = args->maux;
     int j,k,l;
 
-    fprintf(pysamerr,"Alleles to merge at %d\n", pos+1);
+    fprintf(pysam_stderr,"Alleles to merge at %d\n", pos+1);
     for (j=0; j<files->nreaders; j++)
     {
         bcf_sr_t *reader = &files->readers[j];
-        fprintf(pysamerr," reader %d: ", j);
+        fprintf(pysam_stderr," reader %d: ", j);
         for (k=0; k<=reader->nbuffer; k++)
         {
             if ( maux->d[j][k].skip==SKIP_DONE ) continue;
             bcf1_t *line = reader->buffer[k];
             if ( line->pos!=pos ) continue;
-            fprintf(pysamerr,"\t");
-            if ( maux->d[j][k].skip ) fprintf(pysamerr,"[");  // this record will not be merged in this round
+            fprintf(pysam_stderr,"\t");
+            if ( maux->d[j][k].skip ) fprintf(pysam_stderr,"[");  // this record will not be merged in this round
             for (l=0; l<line->n_allele; l++)
-                fprintf(pysamerr,"%s%s", l==0?"":",", line->d.allele[l]);
-            if ( maux->d[j][k].skip ) fprintf(pysamerr,"]");
+                fprintf(pysam_stderr,"%s%s", l==0?"":",", line->d.allele[l]);
+            if ( maux->d[j][k].skip ) fprintf(pysam_stderr,"]");
         }
-        fprintf(pysamerr,"\n");
+        fprintf(pysam_stderr,"\n");
     }
-    fprintf(pysamerr," counts: ");
-    for (j=0; j<maux->nals; j++) fprintf(pysamerr,"%s   %dx %s", j==0?"":",",maux->cnt[j], maux->als[j]); fprintf(pysamerr,"\n");
+    fprintf(pysam_stderr," counts: ");
+    for (j=0; j<maux->nals; j++) fprintf(pysam_stderr,"%s   %dx %s", j==0?"":",",maux->cnt[j], maux->als[j]); fprintf(pysam_stderr,"\n");
     for (j=0; j<files->nreaders; j++)
     {
         bcf_sr_t *reader = &files->readers[j];
-        fprintf(pysamerr," out %d: ", j);
+        fprintf(pysam_stderr," out %d: ", j);
         for (k=0; k<=reader->nbuffer; k++)
         {
             if ( maux->d[j][k].skip==SKIP_DONE ) continue;
             bcf1_t *line = reader->buffer[k];
             if ( line->pos!=pos ) continue;
             if ( maux->d[j][k].skip ) continue;
-            fprintf(pysamerr,"\t");
+            fprintf(pysam_stderr,"\t");
             for (l=0; l<line->n_allele; l++)
-                fprintf(pysamerr,"%s%s", l==0?"":",", maux->als[maux->d[j][k].map[l]]);
+                fprintf(pysam_stderr,"%s%s", l==0?"":",", maux->als[maux->d[j][k].map[l]]);
         }
-        fprintf(pysamerr,"\n");
+        fprintf(pysam_stderr,"\n");
     }
-    fprintf(pysamerr,"\n");
+    fprintf(pysam_stderr,"\n");
 }
 
 // Determine which line should be merged from which reader: go through all
@@ -1915,7 +1915,7 @@ void merge_vcf(args_t *args)
             char buf[10]; snprintf(buf,10,"%d",i+1);
             merge_headers(args->out_hdr, args->files->readers[i].header,buf,args->force_samples);
         }
-        bcf_hdr_append_version(args->out_hdr, args->argc, args->argv, "bcftools_merge");
+        if (args->record_cmd_line) bcf_hdr_append_version(args->out_hdr, args->argc, args->argv, "bcftools_merge");
         bcf_hdr_sync(args->out_hdr);
     }
     info_rules_init(args);
@@ -1950,26 +1950,27 @@ void merge_vcf(args_t *args)
 
 static void usage(void)
 {
-    fprintf(pysamerr, "\n");
-    fprintf(pysamerr, "About:   Merge multiple VCF/BCF files from non-overlapping sample sets to create one multi-sample file.\n");
-    fprintf(pysamerr, "         Note that only records from different files can be merged, never from the same file. For\n");
-    fprintf(pysamerr, "         \"vertical\" merge take a look at \"bcftools norm\" instead.\n");
-    fprintf(pysamerr, "Usage:   bcftools merge [options] <A.vcf.gz> <B.vcf.gz> [...]\n");
-    fprintf(pysamerr, "\n");
-    fprintf(pysamerr, "Options:\n");
-    fprintf(pysamerr, "        --force-samples                resolve duplicate sample names\n");
-    fprintf(pysamerr, "        --print-header                 print only the merged header and exit\n");
-    fprintf(pysamerr, "        --use-header <file>            use the provided header\n");
-    fprintf(pysamerr, "    -f, --apply-filters <list>         require at least one of the listed FILTER strings (e.g. \"PASS,.\")\n");
-    fprintf(pysamerr, "    -i, --info-rules <tag:method,..>   rules for merging INFO fields (method is one of sum,avg,min,max,join) or \"-\" to turn off the default [DP:sum,DP4:sum]\n");
-    fprintf(pysamerr, "    -l, --file-list <file>             read file names from the file\n");
-    fprintf(pysamerr, "    -m, --merge <string>               allow multiallelic records for <snps|indels|both|all|none|id>, see man page for details [both]\n");
-    fprintf(pysamerr, "    -o, --output <file>                write output to a file [standard output]\n");
-    fprintf(pysamerr, "    -O, --output-type <b|u|z|v>        'b' compressed BCF; 'u' uncompressed BCF; 'z' compressed VCF; 'v' uncompressed VCF [v]\n");
-    fprintf(pysamerr, "    -r, --regions <region>             restrict to comma-separated list of regions\n");
-    fprintf(pysamerr, "    -R, --regions-file <file>          restrict to regions listed in a file\n");
-    fprintf(pysamerr, "        --threads <int>                number of extra output compression threads [0]\n");
-    fprintf(pysamerr, "\n");
+    fprintf(pysam_stderr, "\n");
+    fprintf(pysam_stderr, "About:   Merge multiple VCF/BCF files from non-overlapping sample sets to create one multi-sample file.\n");
+    fprintf(pysam_stderr, "         Note that only records from different files can be merged, never from the same file. For\n");
+    fprintf(pysam_stderr, "         \"vertical\" merge take a look at \"bcftools norm\" instead.\n");
+    fprintf(pysam_stderr, "Usage:   bcftools merge [options] <A.vcf.gz> <B.vcf.gz> [...]\n");
+    fprintf(pysam_stderr, "\n");
+    fprintf(pysam_stderr, "Options:\n");
+    fprintf(pysam_stderr, "        --force-samples                resolve duplicate sample names\n");
+    fprintf(pysam_stderr, "        --print-header                 print only the merged header and exit\n");
+    fprintf(pysam_stderr, "        --use-header <file>            use the provided header\n");
+    fprintf(pysam_stderr, "    -f, --apply-filters <list>         require at least one of the listed FILTER strings (e.g. \"PASS,.\")\n");
+    fprintf(pysam_stderr, "    -i, --info-rules <tag:method,..>   rules for merging INFO fields (method is one of sum,avg,min,max,join) or \"-\" to turn off the default [DP:sum,DP4:sum]\n");
+    fprintf(pysam_stderr, "    -l, --file-list <file>             read file names from the file\n");
+    fprintf(pysam_stderr, "    -m, --merge <string>               allow multiallelic records for <snps|indels|both|all|none|id>, see man page for details [both]\n");
+    fprintf(pysam_stderr, "        --no-version                   do not append version and command line to the header\n");
+    fprintf(pysam_stderr, "    -o, --output <file>                write output to a file [standard output]\n");
+    fprintf(pysam_stderr, "    -O, --output-type <b|u|z|v>        'b' compressed BCF; 'u' uncompressed BCF; 'z' compressed VCF; 'v' uncompressed VCF [v]\n");
+    fprintf(pysam_stderr, "    -r, --regions <region>             restrict to comma-separated list of regions\n");
+    fprintf(pysam_stderr, "    -R, --regions-file <file>          restrict to regions listed in a file\n");
+    fprintf(pysam_stderr, "        --threads <int>                number of extra output compression threads [0]\n");
+    fprintf(pysam_stderr, "\n");
     exit(1);
 }
 
@@ -1982,6 +1983,7 @@ int main_vcfmerge(int argc, char *argv[])
     args->output_fname = "-";
     args->output_type = FT_VCF;
     args->n_threads = 0;
+    args->record_cmd_line = 1;
     args->collapse = COLLAPSE_BOTH;
     int regions_is_file = 0;
 
@@ -2000,6 +2002,7 @@ int main_vcfmerge(int argc, char *argv[])
         {"regions",required_argument,NULL,'r'},
         {"regions-file",required_argument,NULL,'R'},
         {"info-rules",required_argument,NULL,'i'},
+        {"no-version",no_argument,NULL,8},
         {NULL,0,NULL,0}
     };
     while ((c = getopt_long(argc, argv, "hm:f:r:R:o:O:i:l:",loptions,NULL)) >= 0) {
@@ -2034,6 +2037,7 @@ int main_vcfmerge(int argc, char *argv[])
             case  2 : args->header_only = 1; break;
             case  3 : args->force_samples = 1; break;
             case  9 : args->n_threads = strtol(optarg, 0, 0); break;
+            case  8 : args->record_cmd_line = 0; break;
             case 'h':
             case '?': usage();
             default: error("Unknown argument: %s\n", optarg);
