@@ -2,7 +2,7 @@
 
 /*  bam_cat.c -- efficiently concatenates bam files.
 
-    Copyright (C) 2008-2009, 2011-2013 Genome Research Ltd.
+    Copyright (C) 2008-2009, 2011-2013, 2015-2016 Genome Research Ltd.
     Modified SAMtools work copyright (C) 2010 Illumina, Inc.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -36,6 +36,8 @@ and modified to perform concatenation by Chris Saunders on behalf of
 Illumina.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -45,6 +47,7 @@ Illumina.
 #include "htslib/sam.h"
 #include "htslib/cram.h"
 #include "htslib/khash.h"
+#include "samtools.h"
 
 KHASH_MAP_INIT_STR(s2i, int)
 
@@ -197,7 +200,7 @@ static bam_hdr_t *cram_cat_check_hdr(int nfn, char * const *fn, const bam_hdr_t
 
         in = sam_open(fn[i], "rc");
         if (in == 0) {
-            fprintf(pysamerr, "[%s] ERROR: fail to open file '%s'.\n", __func__, fn[i]);
+            print_error_errno("cat", "fail to open file '%s'", fn[i]);
             return NULL;
         }
         in_c = in->fp.cram;
@@ -206,7 +209,7 @@ static bam_hdr_t *cram_cat_check_hdr(int nfn, char * const *fn, const bam_hdr_t
         int vmin = cram_minor_vers(in_c);
         if ((vers_maj != -1 && vers_maj != vmaj) ||
             (vers_min != -1 && vers_min != vmin)) {
-            fprintf(pysamerr, "[%s] ERROR: input files have differing version numbers.\n",
+            fprintf(pysam_stderr, "[%s] ERROR: input files have differing version numbers.\n",
                     __func__);
             return NULL;
         }
@@ -226,7 +229,7 @@ static bam_hdr_t *cram_cat_check_hdr(int nfn, char * const *fn, const bam_hdr_t
             int added;
 
             new_rg = hash_s2i_inc(*rg2id, rg2id_in->id[ki], rg2id_in->line[ki], &added);
-            //fprintf(pysamerr, "RG %s: #%d -> #%d\n",
+            //fprintf(pysam_stderr, "RG %s: #%d -> #%d\n",
             //        rg2id_in->id[ki], ki, new_rg);
 
             if (added) {
@@ -242,7 +245,7 @@ static bam_hdr_t *cram_cat_check_hdr(int nfn, char * const *fn, const bam_hdr_t
             }
 
             if (new_rg != ki && rg2id_in->n_id > 1) {
-                fprintf(pysamerr, "[%s] ERROR: Same size @RG lists but differing order / contents\n",
+                fprintf(pysam_stderr, "[%s] ERROR: Same size @RG lists but differing order / contents\n",
                         __func__);
                 return NULL;
             }
@@ -304,15 +307,18 @@ int cram_cat(int nfn, char * const *fn, const bam_hdr_t *h, const char* outcram)
     sprintf(vers, "%d.%d", vers_maj, vers_min);
     out = sam_open(outcram, "wc");
     if (out == 0) {
-        fprintf(pysamerr, "[%s] ERROR: fail to open output file '%s'.\n", __func__, outcram);
-        return 1;
+        print_error_errno("cat", "fail to open output file '%s'", outcram);
+        return -1;
     }
     out_c = out->fp.cram;
     cram_set_option(out_c, CRAM_OPT_VERSION, vers);
-    //fprintf(pysamerr, "Creating cram vers %s\n", vers);
+    //fprintf(pysam_stderr, "Creating cram vers %s\n", vers);
 
     cram_fd_set_header(out_c, sam_hdr_parse_(new_h->text,  new_h->l_text)); // needed?
-    sam_hdr_write(out, new_h);
+    if (sam_hdr_write(out, new_h) < 0) {
+        print_error_errno("cat", "Couldn't write header");
+        return -1;
+    }
 
     for (i = 0; i < nfn; ++i) {
         samFile *in;
@@ -323,7 +329,7 @@ int cram_cat(int nfn, char * const *fn, const bam_hdr_t *h, const char* outcram)
 
         in = sam_open(fn[i], "rc");
         if (in == 0) {
-            fprintf(pysamerr, "[%s] ERROR: fail to open file '%s'.\n", __func__, fn[i]);
+            print_error_errno("cat", "fail to open file '%s'", fn[i]);
             return -1;
         }
         in_c = in->fp.cram;
@@ -367,7 +373,7 @@ int cram_cat(int nfn, char * const *fn, const bam_hdr_t *h, const char* outcram)
             // we need to edit the compression header. IF WE CAN.
             if (new_rg) {
                 int zero = 0;
-                //fprintf(pysamerr, "Transcode RG %d to %d\n", 0, new_rg);
+                //fprintf(pysam_stderr, "Transcode RG %d to %d\n", 0, new_rg);
                 cram_transcode_rg(in_c, out_c, c, 1, &zero, &new_rg);
             } else {
                 int32_t num_slices;
@@ -416,44 +422,56 @@ int cram_cat(int nfn, char * const *fn, const bam_hdr_t *h, const char* outcram)
 
 int bam_cat(int nfn, char * const *fn, const bam_hdr_t *h, const char* outbam)
 {
-    BGZF *fp;
-    uint8_t *buf;
+    BGZF *fp, *in = NULL;
+    uint8_t *buf = NULL;
     uint8_t ebuf[BGZF_EMPTY_BLOCK_SIZE];
     const int es=BGZF_EMPTY_BLOCK_SIZE;
     int i;
 
-    fp = strcmp(outbam, "-")? bgzf_open(outbam, "w") : bgzf_fdopen(fileno(stdout), "w");
+    fp = strcmp(outbam, "-")? bgzf_open(outbam, "w") : bgzf_fdopen(fileno(pysam_stdout), "w");
     if (fp == 0) {
-        fprintf(pysamerr, "[%s] ERROR: fail to open output file '%s'.\n", __func__, outbam);
-        return 1;
+        print_error_errno("cat", "fail to open output file '%s'", outbam);
+        return -1;
+    }
+    if (h) {
+        if (bam_hdr_write(fp, h) < 0) {
+            print_error_errno("cat", "Couldn't write header");
+            goto fail;
+        }
     }
-    if (h) bam_hdr_write(fp, h);
 
     buf = (uint8_t*) malloc(BUF_SIZE);
+    if (!buf) {
+        fprintf(pysam_stderr, "[%s] Couldn't allocate buffer\n", __func__);
+        goto fail;
+    }
     for(i = 0; i < nfn; ++i){
-        BGZF *in;
         bam_hdr_t *old;
         int len,j;
 
         in = strcmp(fn[i], "-")? bgzf_open(fn[i], "r") : bgzf_fdopen(fileno(stdin), "r");
         if (in == 0) {
-            fprintf(pysamerr, "[%s] ERROR: fail to open file '%s'.\n", __func__, fn[i]);
-            return -1;
+            print_error_errno("cat", "fail to open file '%s'", fn[i]);
+            goto fail;
         }
         if (in->is_write) return -1;
 
         old = bam_hdr_read(in);
         if (old == NULL) {
-            fprintf(pysamerr, "[%s] ERROR: couldn't read header for '%s'.\n",
+            fprintf(pysam_stderr, "[%s] ERROR: couldn't read header for '%s'.\n",
                     __func__, fn[i]);
-            bgzf_close(in);
-            return -1;
+            goto fail;
+        }
+        if (h == 0 && i == 0) {
+            if (bam_hdr_write(fp, old) < 0) {
+                print_error_errno("cat", "Couldn't write header");
+                goto fail;
+            }
         }
-        if (h == 0 && i == 0) bam_hdr_write(fp, old);
 
         if (in->block_offset < in->block_length) {
-            bgzf_write(fp, in->uncompressed_block + in->block_offset, in->block_length - in->block_offset);
-            bgzf_flush(fp);
+            if (bgzf_write(fp, in->uncompressed_block + in->block_offset, in->block_length - in->block_offset) < 0) goto write_fail;
+            if (bgzf_flush(fp) != 0) goto write_fail;
         }
 
         j=0;
@@ -461,17 +479,20 @@ int bam_cat(int nfn, char * const *fn, const bam_hdr_t *h, const char* outbam)
             if(len<es){
                 int diff=es-len;
                 if(j==0) {
-                    fprintf(pysamerr, "[%s] ERROR: truncated file?: '%s'.\n", __func__, fn[i]);
-                    return -1;
+                    fprintf(pysam_stderr, "[%s] ERROR: truncated file?: '%s'.\n", __func__, fn[i]);
+                    goto fail;
                 }
-                bgzf_raw_write(fp, ebuf, len);
+                if (bgzf_raw_write(fp, ebuf, len) < 0) goto write_fail;
+
                 memcpy(ebuf,ebuf+len,diff);
                 memcpy(ebuf+diff,buf,len);
             } else {
-                if(j!=0) bgzf_raw_write(fp, ebuf, es);
+                if(j!=0) {
+                    if (bgzf_raw_write(fp, ebuf, es) < 0) goto write_fail;
+                }
                 len-= es;
                 memcpy(ebuf,buf+len,es);
-                bgzf_raw_write(fp, buf, len);
+                if (bgzf_raw_write(fp, buf, len) < 0) goto write_fail;
             }
             j=1;
         }
@@ -482,17 +503,29 @@ int bam_cat(int nfn, char * const *fn, const bam_hdr_t *h, const char* outbam)
             const uint8_t gzip2=ebuf[1];
             const uint32_t isize=*((uint32_t*)(ebuf+es-4));
             if(((gzip1!=GZIPID1) || (gzip2!=GZIPID2)) || (isize!=0)) {
-                fprintf(pysamerr, "[%s] WARNING: Unexpected block structure in file '%s'.", __func__, fn[i]);
-                fprintf(pysamerr, " Possible output corruption.\n");
-                bgzf_raw_write(fp, ebuf, es);
+                fprintf(pysam_stderr, "[%s] WARNING: Unexpected block structure in file '%s'.", __func__, fn[i]);
+                fprintf(pysam_stderr, " Possible output corruption.\n");
+                if (bgzf_raw_write(fp, ebuf, es) < 0) goto write_fail;
             }
         }
         bam_hdr_destroy(old);
         bgzf_close(in);
+        in = NULL;
     }
     free(buf);
-    bgzf_close(fp);
+    if (bgzf_close(fp) < 0) {
+        fprintf(pysam_stderr, "[%s] Error on closing '%s'.\n", __func__, outbam);
+        return -1;
+    }
     return 0;
+
+ write_fail:
+    fprintf(pysam_stderr, "[%s] Error writing to '%s'.\n", __func__, outbam);
+ fail:
+    if (in) bgzf_close(in);
+    if (fp) bgzf_close(fp);
+    free(buf);
+    return -1;
 }
 
 
@@ -500,7 +533,7 @@ int main_cat(int argc, char *argv[])
 {
     bam_hdr_t *h = 0;
     char *outfn = 0;
-    int c, ret;
+    int c, ret = 0;
     samFile *in;
 
     while ((c = getopt(argc, argv, "h:o:")) >= 0) {
@@ -508,12 +541,12 @@ int main_cat(int argc, char *argv[])
             case 'h': {
                 samFile *fph = sam_open(optarg, "r");
                 if (fph == 0) {
-                    fprintf(pysamerr, "[%s] ERROR: fail to read the header from '%s'.\n", __func__, argv[1]);
+                    fprintf(pysam_stderr, "[%s] ERROR: fail to read the header from '%s'.\n", __func__, argv[1]);
                     return 1;
                 }
                 h = sam_hdr_read(fph);
                 if (h == NULL) {
-                    fprintf(pysamerr,
+                    fprintf(pysam_stderr,
                             "[%s] ERROR: failed to read the header for '%s'.\n",
                             __func__, argv[1]);
                     return 1;
@@ -525,30 +558,32 @@ int main_cat(int argc, char *argv[])
         }
     }
     if (argc - optind < 1) {
-        fprintf(pysamerr, "Usage: samtools cat [-h header.sam] [-o out.bam] <in1.bam> [...]\n");
+        fprintf(pysam_stderr, "Usage: samtools cat [-h header.sam] [-o out.bam] <in1.bam> [...]\n");
         return 1;
     }
 
     in = sam_open(argv[optind], "r");
     if (!in) {
-        fprintf(pysamerr, "[%s] ERROR: failed to open file '%s'.\n", __func__, argv[optind]);
+        print_error_errno("cat", "failed to open file '%s'", argv[optind]);
         return 1;
     }
 
     switch (hts_get_format(in)->format) {
     case bam:
         sam_close(in);
-        ret = bam_cat(argc - optind, argv + optind, h, outfn? outfn : "-");
+        if (bam_cat(argc - optind, argv + optind, h, outfn? outfn : "-") < 0)
+            ret = 1;
         break;
 
     case cram:
         sam_close(in);
-        ret = cram_cat(argc - optind, argv + optind, h, outfn? outfn : "-");
+        if (cram_cat(argc - optind, argv + optind, h, outfn? outfn : "-") < 0)
+            ret = 1;
         break;
 
     default:
         sam_close(in);
-        fprintf(pysamerr, "[%s] ERROR: input is not BAM or CRAM\n", __func__);
+        fprintf(pysam_stderr, "[%s] ERROR: input is not BAM or CRAM\n", __func__);
         return 1;
     }
     free(outfn);
