@@ -25,6 +25,8 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.  */
 
+#include <config.h>
+
 #include <string.h>
 #include <unistd.h>
 #include "htslib/faidx.h"
@@ -33,7 +35,7 @@ DEALINGS IN THE SOFTWARE.  */
 int samthreads(samfile_t *fp, int n_threads, int n_sub_blks)
 {
     if (hts_get_format(fp->file)->format != bam || !fp->is_write) return -1;
-    bgzf_mt(fp->x.bam, n_threads, n_sub_blks);
+    if (bgzf_mt(fp->x.bam, n_threads, n_sub_blks) < 0) return -1;
     return 0;
 }
 
@@ -44,6 +46,10 @@ samfile_t *samopen(const char *fn, const char *mode, const void *aux)
     if (hts_fp == NULL)  return NULL;
 
     samfile_t *fp = malloc(sizeof (samfile_t));
+    if (!fp) {
+        sam_close(hts_fp);
+        return NULL;
+    }
     fp->file = hts_fp;
     fp->x.bam = hts_fp->fp.bgzf;
     if (strchr(mode, 'r')) {
@@ -62,13 +68,21 @@ samfile_t *samopen(const char *fn, const char *mode, const void *aux)
         }
         fp->is_write = 0;
         if (fp->header->n_targets == 0 && bam_verbose >= 1)
-            fprintf(pysamerr, "[samopen] no @SQ lines in the header.\n");
+            fprintf(pysam_stderr, "[samopen] no @SQ lines in the header.\n");
     }
     else {
         enum htsExactFormat fmt = hts_get_format(fp->file)->format;
         fp->header = (bam_hdr_t *)aux;  // For writing, we won't free it
         fp->is_write = 1;
-        if (!(fmt == text_format || fmt == sam) || strchr(mode, 'h')) sam_hdr_write(fp->file, fp->header);
+        if (!(fmt == text_format || fmt == sam) || strchr(mode, 'h')) {
+            if (sam_hdr_write(fp->file, fp->header) < 0) {
+                if (bam_verbose >= 1)
+                    fprintf(pysam_stderr, "[samopen] Couldn't write header\n");
+                sam_close(hts_fp);
+                free(fp);
+                return NULL;
+            }
+        }
     }
 
     return fp;
@@ -122,11 +136,11 @@ char *samfaipath(const char *fn_ref)
     strcat(strcpy(fn_list, fn_ref), ".fai");
     if (access(fn_list, R_OK) == -1) { // fn_list is unreadable
         if (access(fn_ref, R_OK) == -1) {
-            fprintf(pysamerr, "[samfaipath] fail to read file %s.\n", fn_ref);
+            fprintf(pysam_stderr, "[samfaipath] fail to read file %s.\n", fn_ref);
         } else {
-            if (bam_verbose >= 3) fprintf(pysamerr, "[samfaipath] build FASTA index...\n");
+            if (bam_verbose >= 3) fprintf(pysam_stderr, "[samfaipath] build FASTA index...\n");
             if (fai_build(fn_ref) == -1) {
-                fprintf(pysamerr, "[samfaipath] fail to build FASTA index.\n");
+                fprintf(pysam_stderr, "[samfaipath] fail to build FASTA index.\n");
                 free(fn_list); fn_list = 0;
             }
         }
