@@ -29,7 +29,8 @@ from TestUtils import checkBinaryEqual, checkURL, \
     get_temp_filename
 
 
-DATADIR = "pysam_data"
+DATADIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
+                                       "pysam_data"))
 
 
 ##################################################
@@ -353,26 +354,53 @@ class BasicTestBAMFromFilename(BasicTestBAMFromFetch):
 class BasicTestBAMFromFile(BasicTestBAMFromFetch):
 
     def setUp(self):
-        f = open(os.path.join(DATADIR, "ex3.bam"))
-        self.samfile = pysam.AlignmentFile(
-            f, "rb")
+        with open(os.path.join(DATADIR, "ex3.bam")) as f:
+            self.samfile = pysam.AlignmentFile(
+                f, "rb")
+        self.reads = [r for r in self.samfile]
+
+
+class BasicTestBAMFromFileNo(BasicTestBAMFromFetch):
+
+    def setUp(self):
+        with open(os.path.join(DATADIR, "ex3.bam")) as f:
+            self.samfile = pysam.AlignmentFile(
+                f.fileno(), "rb")
         self.reads = [r for r in self.samfile]
 
 
 class BasicTestSAMFromFile(BasicTestBAMFromFetch):
 
     def setUp(self):
-        f = open(os.path.join(DATADIR, "ex3.sam"))
-        self.samfile = pysam.AlignmentFile(
-            f, "r")
+        with open(os.path.join(DATADIR, "ex3.sam")) as f:
+            self.samfile = pysam.AlignmentFile(
+                f, "r")
+        self.reads = [r for r in self.samfile]
+
+
+class BasicTestSAMFromFileNo(BasicTestBAMFromFetch):
+
+    def setUp(self):
+        with open(os.path.join(DATADIR, "ex3.sam")) as f:
+            self.samfile = pysam.AlignmentFile(
+                f.fileno(), "r")
         self.reads = [r for r in self.samfile]
 
 
 class BasicTestCRAMFromFile(BasicTestCRAMFromFetch):
 
     def setUp(self):
-        f = open(os.path.join(DATADIR, "ex3.cram"))
-        self.samfile = pysam.AlignmentFile(f, "rc")
+        with open(os.path.join(DATADIR, "ex3.cram")) as f:
+            self.samfile = pysam.AlignmentFile(f, "rc")
+        self.reads = [r for r in self.samfile]
+
+
+class BasicTestCRAMFromFileNo(BasicTestCRAMFromFetch):
+
+    def setUp(self):
+        with open(os.path.join(DATADIR, "ex3.cram")) as f:
+            self.samfile = pysam.AlignmentFile(
+                f.fileno(), "rc")
         self.reads = [r for r in self.samfile]
 
 
@@ -690,7 +718,7 @@ class TestIO(unittest.TestCase):
         samfile = pysam.AlignmentFile(f, "rb")
         f.close()
         self.assertTrue(f.closed)
-        # access to Samfile should still work
+        # access to Samfile still works
         self.checkEcho("ex1.bam",
                        "ex1.bam",
                        "tmp_ex1.bam",
@@ -818,6 +846,15 @@ class TestIO(unittest.TestCase):
             mode="rb")
         self.assertEqual(len(list(samfile.fetch())), 3270)
 
+    def testBAMWithCSIIndex(self):
+        '''see issue 116'''
+        input_filename = os.path.join(DATADIR, "ex1_csi.bam")
+        samfile = pysam.AlignmentFile(input_filename,
+                                      "rb",
+                                      check_sq=False)
+        samfile.fetch('chr2')
+
+
 
 class TestAutoDetect(unittest.TestCase):
 
@@ -1291,8 +1328,8 @@ class TestHeaderSAM(unittest.TestCase):
 
     """testing header manipulation"""
 
-    header = {'SQ': [{'LN': 1575, 'SN': 'chr1'},
-                     {'LN': 1584, 'SN': 'chr2'}],
+    header = {'SQ': [{'LN': 1575, 'SN': 'chr1', 'AH': 'chr1:5000000-5010000'},
+                     {'LN': 1584, 'SN': 'chr2', 'AH': '*'}],
               'RG': [{'LB': 'SC_1', 'ID': 'L1', 'SM': 'NA12891',
                       'PU': 'SC_1_10', "CN": "name:with:colon"},
                      {'LB': 'SC_2', 'ID': 'L2', 'SM': 'NA12891',
@@ -2343,6 +2380,46 @@ class TestPileupQueryPosition(unittest.TestCase):
                         last[r.alignment.query_name] = r.query_position
 
 
+class TestFindIntrons(unittest.TestCase):
+    samfilename = "pysam_data/ex_spliced.bam"
+
+    def setUp(self):
+        self.samfile = pysam.AlignmentFile(self.samfilename)
+
+    def tearDown(self):
+        self.samfile.close()
+
+    def test_total(self):
+        all_read_counts = self.samfile.count()
+        splice_sites = self.samfile.find_introns(self.samfile.fetch())
+        self.assertEqual(sum(splice_sites.values()), all_read_counts -1)  # there is a single unspliced read in there
+         
+    def test_first(self):
+        reads = list(self.samfile.fetch())[:10]
+        splice_sites = self.samfile.find_introns(reads)
+        starts = [14792+38 - 1]
+        stops = [14792+38 + 140 - 1]
+        self.assertEqual(len(splice_sites), 1)
+        self.assertTrue((starts[0], stops[0]) in splice_sites)
+        self.assertEqual(splice_sites[(starts[0], stops[0])], 9) # first one is the unspliced read
+
+    def test_all(self):
+        reads = list(self.samfile.fetch())
+        splice_sites = self.samfile.find_introns(reads)
+        should = collections.Counter({
+            (14829, 14969): 33,
+            (15038, 15795): 24,
+            (15947, 16606): 3,
+            (16765, 16857): 9,
+            (16765, 16875): 1,
+            (17055, 17232): 19,
+            (17055, 17605): 3,
+            (17055, 17914): 1,
+            (17368, 17605): 7,
+            })
+        self.assertEqual(should,  splice_sites)
+
+
 class TestLogging(unittest.TestCase):
 
     '''test around bug issue 42,
@@ -2511,7 +2588,6 @@ class TestMappedUnmapped(unittest.TestCase):
                              inf.mapped)
 
 
-
 class TestSamtoolsProxy(unittest.TestCase):
 
     '''tests for sanity checking access to samtools functions.'''
@@ -2592,6 +2668,34 @@ class TestVerbosity(unittest.TestCase):
         self.assertEqual(pysam.get_verbosity(), 3)
 
 
+class TestSanityCheckingBAM(unittest.TestCase):
+    
+    mode = "wb"
+
+    def check_write(self, read):
+        
+        fn = "tmp_test_sanity_check.bam"
+        names = ["chr1"]
+        lengths = [10000]
+        with pysam.AlignmentFile(
+                fn, 
+                self.mode,
+                reference_names=names,
+                reference_lengths=lengths) as outf:
+            outf.write(read)
+
+        if os.path.exists(fn):
+            os.unlink(fn)
+            
+    def test_empty_read_gives_value_error(self):
+        read = pysam.AlignedSegment()
+        self.check_write(read)
+
+# SAM writing fails, as query length is 0
+# class TestSanityCheckingSAM(TestSanityCheckingSAM):
+#     mode = "w"
+    
+
 if __name__ == "__main__":
     # build data files
     print ("building data files")
