@@ -1,8 +1,15 @@
 import os
+import sys
 import unittest
 import pysam
 import gzip
 import subprocess
+
+try:
+    from pathlib import Path
+except ImportError:
+    Path = None
+
 from TestUtils import get_temp_filename, check_lines_equal
 
 DATADIR="cbcf_data"
@@ -75,6 +82,17 @@ class TestOpening(unittest.TestCase):
 
         os.unlink("tmp_testEmptyFile.vcf")
 
+
+    if Path and sys.version_info >= (3,6):
+        def testEmptyFileVCFFromPath(self):
+            with open("tmp_testEmptyFile.vcf", "w"):
+                pass
+
+            self.assertRaises(ValueError, pysam.VariantFile,
+                              Path("tmp_testEmptyFile.vcf"))
+
+            os.unlink("tmp_testEmptyFile.vcf")
+
     def testEmptyFileVCFGZWithIndex(self):
         with open("tmp_testEmptyFile.vcf", "w"):
             pass
@@ -171,12 +189,12 @@ class TestHeader(unittest.TestCase):
         # remove last header line starting with #CHROM
         ref.pop()
         ref = sorted(ref)
-        comp = sorted([str(x) for x in v.header.records])
+        comp = sorted(str(x) for x in v.header.records)
 
         self.assertEqual(len(ref), len(comp))
 
         for x, y in zip(ref, comp):
-            self.assertEqual(x[:-1], str(y))
+            self.assertEqual(x, y)
 
 
 # These tests need to be separate and start from newly opened files.  This
@@ -195,6 +213,13 @@ class TestParsing(unittest.TestCase):
         chrom = [rec.chrom for rec in v]
         self.assertEqual(chrom, ['M', '17', '20', '20', '20'])
 
+    if Path and sys.version_info >= (3,6):
+        def testChromFromPath(self):
+            fn = os.path.join(DATADIR, self.filename)
+            v = pysam.VariantFile(Path(fn))
+            chrom = [rec.chrom for rec in v]
+            self.assertEqual(chrom, ['M', '17', '20', '20', '20'])
+
     def testPos(self):
         fn = os.path.join(DATADIR, self.filename)
         v = pysam.VariantFile(fn)
@@ -330,9 +355,22 @@ class TestConstructionVCFWithContigs(unittest.TestCase):
     """construct VariantFile from scratch."""
 
     filename = "example_vcf42_withcontigs.vcf"
+    compression = 'NONE'
+    description = 'VCF version 4.2 variant calling text'
+
+    def testBase(self):
+        with pysam.VariantFile(os.path.join(DATADIR, self.filename)) as inf:
+            self.assertEqual(inf.category, 'VARIANTS')
+            self.assertEqual(inf.format, 'VCF')
+            self.assertEqual(inf.version, (4, 2))
+            self.assertEqual(inf.compression, self.compression)
+            self.assertEqual(inf.description, self.description)
+            self.assertTrue(inf.is_open)
+            self.assertEqual(inf.is_read, True)
+            self.assertEqual(inf.is_write, False)
 
     def complete_check(self, fn_in, fn_out):
-
+        self.maxDiff = None
         check_lines_equal(
             self, fn_in, fn_out, sort=True,
             filter_f=lambda x: x.startswith("##contig"))
@@ -349,14 +387,15 @@ class TestConstructionVCFWithContigs(unittest.TestCase):
         for record in vcf_in.header.records:
             header.add_record(record)
 
-        fn = str("tmp_VariantFileTest_testConstructionWithRecords") + ".vcf"
-        vcf_out = pysam.VariantFile(fn, "w", header=header)
-        for record in vcf_in:
-            # currently segfaults here:
-            # vcf_out.write(record)
-            pass
-        return
+        for sample in vcf_in.header.samples:
+            header.add_sample(sample)
 
+        vcf_out = pysam.VariantFile(fn_out, "w", header=header)
+        for record in vcf_in:
+            record.translate(header)
+            vcf_out.write(record)
+
+        vcf_in.close()
         vcf_out.close()
         self.complete_check(fn_in, fn_out)
 
@@ -370,6 +409,7 @@ class TestConstructionVCFWithContigs(unittest.TestCase):
         for record in vcf_in:
             vcf_out.write(record)
 
+        vcf_in.close()
         vcf_out.close()
 
         self.complete_check(fn_in, fn_out)
@@ -397,8 +437,8 @@ class TestConstructionVCFWithContigs(unittest.TestCase):
 
         self.complete_check(fn_in, fn_out)
 
-# Currently segfaults for VCFs without contigs
-# class TestConstructionVCFWithoutContigs(TestConstructionVCFWithContigs):
+
+#class TestConstructionVCFWithoutContigs(TestConstructionVCFWithContigs):
 #     """construct VariantFile from scratch."""
 #     filename = "example_vcf40.vcf"
 
@@ -407,18 +447,33 @@ class TestConstructionVCFGZWithContigs(TestConstructionVCFWithContigs):
     """construct VariantFile from scratch."""
 
     filename = "example_vcf42_withcontigs.vcf.gz"
+    compression = 'BGZF'
+    description = 'VCF version 4.2 BGZF-compressed variant calling data'
 
 
 class TestConstructionVCFGZWithoutContigs(TestConstructionVCFWithContigs):
     """construct VariantFile from scratch."""
 
     filename = "example_vcf42.vcf.gz"
+    compression = 'BGZF'
+    description = 'VCF version 4.2 BGZF-compressed variant calling data'
 
 
 class TestSettingRecordValues(unittest.TestCase):
 
     filename = "example_vcf40.vcf"
 
+    def testBase(self):
+        with pysam.VariantFile(os.path.join(DATADIR, self.filename)) as inf:
+            self.assertEqual(inf.category, 'VARIANTS')
+            self.assertEqual(inf.format, 'VCF')
+            self.assertEqual(inf.version, (4, 0))
+            self.assertEqual(inf.compression, 'NONE')
+            self.assertEqual(inf.description, 'VCF version 4.0 variant calling text')
+            self.assertTrue(inf.is_open)
+            self.assertEqual(inf.is_read, True)
+            self.assertEqual(inf.is_write, False)
+
     def testSetQual(self):
         with pysam.VariantFile(os.path.join(DATADIR, self.filename)) as inf:
             record = next(inf)
@@ -435,8 +490,7 @@ class TestSettingRecordValues(unittest.TestCase):
             sample = record.samples["NA00001"]
             print (sample["GT"])
             self.assertEqual(sample["GT"], (0, 0))
-#	Fails with TypeError
-#            sample["GT"] = sample["GT"]
+            sample["GT"] = sample["GT"]
 
 class TestSubsetting(unittest.TestCase):
     
